Package: quantspec
Version: 1.2-4
Encoding: UTF-8
Date: 2024-07-10
Title: Quantile-Based Spectral Analysis of Time Series
Authors@R: c(person("Tobias", "Kley", role=c("aut","cre"),
    email = "tobias.kley@uni-goettingen.de"),
    person("Stefan", "Birr",  role = c("ctb"), 
	comment = "Contributions to lag window estimation", email = "stefan.birr@rub.de"))
Depends: R (>= 3.0.0), stats4
Suggests: testthat
Imports: methods, graphics, quantreg, abind, zoo, snowfall, Rcpp (>=
        0.11.0)
Description: Methods to determine, smooth and plot quantile periodograms for
    univariate and multivariate time series. See Kley (2016) <doi:10.18637/jss.v070.i03>
    for a description and tutorial.
License: GPL (>= 2)
URL: https://github.com/tobiaskley/quantspec
BugReports: https://github.com/tobiaskley/quantspec/issues
LazyData: TRUE
LinkingTo: Rcpp
Collate: 'Class-BootPos.R' 'generics.R' 'Class-LagOperator.R'
        'Class-ClippedCov.R' 'Class-QSpecQuantity.R' 'aux-functions.R'
        'Class-FreqRep.R' 'Class-ClippedFT.R' 'Class-QuantileSD.R'
        'Class-IntegrQuantileSD.R' 'Class-Weight.R'
        'Class-KernelWeight.R' 'Class-LagEstimator.R' 'kernels.R'
        'Class-LagKernelWeight.R' 'Class-MovingBlocks.R'
        'Class-QRegEstimator.R' 'Class-QuantilePG.R'
        'Class-SmoothedPG.R' 'Class-SpecDistrWeight.R' 'RcppExports.R'
        'data.R' 'deprecated.R' 'models.R' 'quantspec-package.R'
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2024-07-10 13:05:01 UTC; tkley
Author: Tobias Kley [aut, cre],
  Stefan Birr [ctb] (Contributions to lag window estimation)
Maintainer: Tobias Kley <tobias.kley@uni-goettingen.de>
Repository: CRAN
Date/Publication: 2024-07-11 12:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 09:45:08 UTC; unix
Archs: quantspec.so.dSYM
