# Generated by roxygen2: do not edit by hand

export(AR1)
export(AR2)
export(ARCH1)
export(QAR1)
export(W0)
export(W1)
export(W2)
export(W3)
export(WDaniell)
export(WParzen)
export(clippedCov)
export(clippedFT)
export(closest.pos)
export(frequenciesValidator)
export(getB)
export(getBootPos)
export(getBw)
export(getCoherency)
export(getCoherencySdNaive)
export(getDescr)
export(getFreqRep)
export(getFrequencies)
export(getIsRankBased)
export(getLagOperator)
export(getLevels)
export(getMaxLag)
export(getMeanPG)
export(getN)
export(getParallel)
export(getPointwiseCIs)
export(getPositions)
export(getQuantilePG)
export(getQuantileSD)
export(getR)
export(getSdBoot)
export(getSdNaive)
export(getStdError)
export(getTs)
export(getType)
export(getValues)
export(getW)
export(getWeight)
export(getWnj)
export(getY)
export(increasePrecision)
export(integrQuantileSD)
export(kernelWeight)
export(lagEstimator)
export(lagKernelWeight)
export(lenTS)
export(movingBlocks)
export(qRegEstimator)
export(quantilePG)
export(quantileSD)
export(smoothedPG)
export(specDistrWeight)
export(timeSeriesValidator)
export(ts1)
export(ts2)
export(ts3)
exportClasses(BootPos)
exportClasses(ClippedFT)
exportClasses(FreqRep)
exportClasses(IntegrQuantileSD)
exportClasses(KernelWeight)
exportClasses(LagEstimator)
exportClasses(LagKernelWeight)
exportClasses(MovingBlocks)
exportClasses(QRegEstimator)
exportClasses(QSpecQuantity)
exportClasses(QuantilePG)
exportClasses(QuantileSD)
exportClasses(SmoothedPG)
exportClasses(SpecDistrWeight)
exportMethods(plot)
import(graphics)
import(methods)
import(stats4)
importFrom(Rcpp,evalCpp)
importFrom(abind,abind)
importFrom(grDevices,gray)
importFrom(quantreg,rq)
importFrom(snowfall,sfExportAll)
importFrom(snowfall,sfInit)
importFrom(snowfall,sfLapply)
importFrom(snowfall,sfLibrary)
importFrom(snowfall,sfStop)
importFrom(stats,acf)
importFrom(stats,convolve)
importFrom(stats,integrate)
importFrom(stats,is.ts)
importFrom(stats,mvfft)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,smooth.spline)
importFrom(utils,head)
importFrom(zoo,coredata)
importFrom(zoo,is.zoo)
useDynLib(quantspec)
